/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.farm.fieldplan;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.logging.Level;
import java.util.logging.Logger;
import nl.cloudfarming.client.farm.model.CropField;
import nl.cloudfarming.client.farm.model.CropFieldManager;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;
import org.openide.loaders.DataObjectNotFoundException;
import org.openide.util.Exceptions;
import org.openide.util.lookup.ServiceProvider;

/**
 *
 * @author Timon Veenstra <monezz@gmail.com>
 */
@ServiceProvider(service=CropFieldManager.class)
public class CropFieldManagerImpl implements CropFieldManager {

    private static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.farm.fieldplan.CropFieldManagerImpl");
    
    @Override
    public CropField loadCropField(FileObject cropFieldObject) throws IOException {
        LOGGER.log(Level.FINEST, "loading CropField data from  {0}", cropFieldObject.getName());
        CropField cropField = null;
        try (ObjectInputStream ois = new ObjectInputStream(cropFieldObject.getInputStream())) {
            cropField = (CropField) ois.readObject();
        } catch (ClassNotFoundException | IOException ex) {
            Exceptions.printStackTrace(ex);
            throw new DataObjectNotFoundException(cropFieldObject);
        }
        return cropField;
    }

    @Override
    public void save(CropField cropField, FileObject productionUnitFolder) throws IOException {
        LOGGER.log(Level.FINEST, "saving CropField data to folder {0}", productionUnitFolder.getName());
        String name = (cropField.getName() == null) ? new Long(cropField.getId()).toString() : cropField.getName();

        String cropFieldFileName = FileUtil.findFreeFileName(productionUnitFolder, name, CropField.EXT);
        LOGGER.log(Level.FINEST, "found free filename {0} for cropField", cropFieldFileName);

        try (ObjectOutputStream oos = new ObjectOutputStream(productionUnitFolder.createData(cropFieldFileName, CropField.EXT).getOutputStream())) {
            oos.writeObject(cropField);
        }
        // LinkageError: http://frankkieviet.blogspot.com/2009/03/javalanglinkageerror-loader-constraint.html
    }    
}
