/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.farm.fieldplan;

import java.awt.Image;
import java.awt.datatransfer.Transferable;
import java.awt.event.ActionEvent;
import java.beans.BeanInfo;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import javax.swing.AbstractAction;
import javax.swing.Action;
import nl.cloudfarming.client.farm.model.CropProductionUnit;
import nl.cloudfarming.client.farm.model.FieldPlan;
import org.netbeans.core.api.multiview.MultiViews;
import org.openide.filesystems.FileObject;
import org.openide.nodes.FilterNode;
import org.openide.nodes.Node;
import org.openide.util.ImageUtilities;
import org.openide.util.NbBundle;
import org.openide.util.datatransfer.PasteType;
import org.openide.util.lookup.Lookups;
import org.openide.util.lookup.ProxyLookup;
import org.openide.windows.TopComponent;

/**
 *
 * @author Timon Veenstra <monezz@gmail.com>
 */
public class CropProductionUnitNode extends FilterNode implements Serializable{

    public CropProductionUnitNode(CropProductionUnit cropProductionUnit, Node original) {
        super(original, new CropProductionUnitChildren(original),new ProxyLookup(original.getLookup(), Lookups.fixed(cropProductionUnit)));
    }
    
    @Override
    public Image getIcon(int type) {
        switch (type) {
            case BeanInfo.ICON_COLOR_32x32:
            case BeanInfo.ICON_MONO_32x32:
                return ImageUtilities.loadImage("nl/cloudfarming/client/farm/fieldplan/plant-icon.png32");
            default:
                return ImageUtilities.loadImage("nl/cloudfarming/client/farm/fieldplan/plant-icon.png");
        }
    }

    @Override
    public Action[] getActions(boolean context) {
        Action[] actions = new Action[1];
        actions[0] = new OpenCropProductionUnitDetailsAction();
        return actions;
    }
    
    
    
   @Override
    public boolean canCopy() {
        return false;
    }

    @Override
    public boolean canCut() {
        return false;
    }

    @Override
    public PasteType getDropType(Transferable t, int action, int index) {
        return null;
    }

    @Override
    public Image getOpenedIcon(int type) {
        return getIcon(type);
    }    
    
    private static class CropProductionUnitChildren extends FilterNode.Children {

        public CropProductionUnitChildren(Node or) {
            super(or);
        }
        @Override
        protected Node[] createNodes(Node object) {
            List<Node> result = new ArrayList<>();
            for (Node node : super.createNodes(object)) {
                if (accept(node)) {
                    result.add(new FilterNode(node));
                }
            }
            return result.toArray(new Node[0]);
        }

        private boolean accept(Node node) {
            FileObject fileObject = node.getLookup().lookup(FileObject.class);
            if (FieldPlanManagerImpl.PRODUCTION_UNIT_OBJECT_NAME.equals(fileObject.getName())) {
                return false;
            }
            if (fileObject == null){
                return false;
            }
            return true;
        }       
    }
    
    @NbBundle.Messages(value = "CropProductionUnit open details action name=Open")
    private final class OpenCropProductionUnitDetailsAction extends AbstractAction {

        public OpenCropProductionUnitDetailsAction() {
            super(Bundle.CropProductionUnit_open_details_action_name());
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            TopComponent tc = MultiViews.createMultiView(CropProductionUnit.MIME_TYPE, CropProductionUnitNode.this);
            tc.open();
            tc.requestActive();
        }
    }    
}
