/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.farm.fieldplan;

import nl.cloudfarming.client.farm.model.FieldPlanManager;
import java.awt.Image;
import java.awt.datatransfer.Transferable;
import java.awt.event.ActionEvent;
import java.beans.BeanInfo;
import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.AbstractAction;
import javax.swing.Action;
import nl.cloudfarming.client.farm.model.CropProductionUnit;
import nl.cloudfarming.client.farm.model.FieldPlan;
import org.netbeans.core.api.multiview.MultiViews;
import org.openide.filesystems.FileObject;
import org.openide.nodes.FilterNode;
import org.openide.nodes.Node;
import org.openide.util.*;
import org.openide.util.datatransfer.PasteType;
import org.openide.util.lookup.Lookups;
import org.openide.util.lookup.ProxyLookup;
import org.openide.windows.TopComponent;

/**
 * FieldPlanNode
 *
 * Filternode around the data folder node containing the fieldplan.
 *
 *
 * @author Timon Veenstra <monezz@gmail.com>
 */
public class FieldPlanNode extends FilterNode implements Lookup.Provider, Serializable {

    public FieldPlanNode(FieldPlan fieldPlan, Node original) {
        super(original, new FieldPlanChildren(original), new ProxyLookup(original.getLookup(), Lookups.fixed(fieldPlan)));
    }

    @Override
    public Image getIcon(int type) {
        switch (type) {
            case BeanInfo.ICON_COLOR_32x32:
            case BeanInfo.ICON_MONO_32x32:
                return ImageUtilities.loadImage("nl/cloudfarming/client/farm/fieldplan/plant-icon.png32");
            default:
                return ImageUtilities.loadImage("nl/cloudfarming/client/farm/fieldplan/plant-icon.png");
        }
    }

    @Override
    public Action[] getActions(boolean context) {
        Action[] actions = new Action[3];
        actions[0] = new OpenFieldPlanDetailsAction();
        actions[1] = null;
        actions[2] = new FieldPlanDeleteAction();
        return actions;

    }

    @Override
    public boolean canCopy() {
        return false;
    }

    @Override
    public boolean canCut() {
        return false;
    }

    @Override
    public PasteType getDropType(Transferable t, int action, int index) {
        return null;
    }

    @Override
    public Image getOpenedIcon(int type) {
        return getIcon(type);
    }

    public static class FieldPlanChildren extends FilterNode.Children {

        public FieldPlanChildren(Node or) {
            super(or);
        }

        @Override
        protected Node[] createNodes(Node object) {
            List<Node> result = new ArrayList<>();
            for (Node node : super.createNodes(object)) {
                if (accept(node)) {
                    // retrieve the fieldplan manager from the fieldplan node
                    FieldPlanManager fpm = getNode().getParentNode().getLookup().lookup(FieldPlanManager.class);
                    FileObject fileObject = node.getLookup().lookup(FileObject.class);
                    try {
                        CropProductionUnit cropProductionUnit = fpm.loadCropProductionUnit(fileObject);
                        if (cropProductionUnit != null){
                            result.add(new CropProductionUnitNode(cropProductionUnit, node));
                        }
                    } catch (IOException ex) {
                        Exceptions.printStackTrace(ex);
                    }

                }
            }
            return result.toArray(new Node[0]);
        }

        private boolean accept(Node node) {
            FileObject fileObject = node.getLookup().lookup(FileObject.class);
            if (FieldPlanManagerImpl.FIELDPLAN_OBJECT_NAME.equals(fileObject.getName())) {
                return false;
            }
            if (fileObject == null){
                return false;
            }
            return true;
        }
    }

    @NbBundle.Messages({"FieldPlanDeleteAction.Action_name=Delete", "FieldPlanDeleteAction.unable_to_delete=Unable to delete file "})
    private static class FieldPlanDeleteAction extends AbstractAction {

        private static final Logger LOG = Logger.getLogger("nl.cloudfarming.client.farm.fieldplan");

        public FieldPlanDeleteAction() {
            super(Bundle.FieldPlanDeleteAction_Action_name());
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            FileObject fieldPlanFolder = Utilities.actionsGlobalContext().lookup(FileObject.class);
            try {
                fieldPlanFolder.delete();
            } catch (IOException ex) {
                LOG.log(Level.WARNING, "{0} {1} {2}", new Object[]{Bundle.FieldPlanDeleteAction_unable_to_delete(), fieldPlanFolder.getName(), ex});
            }
        }
    }

    @NbBundle.Messages(value = "Field Plan open details action name=Open")
    private final class OpenFieldPlanDetailsAction extends AbstractAction {

        public OpenFieldPlanDetailsAction() {
            super(Bundle.Field_Plan_open_details_action_name());
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            TopComponent tc = MultiViews.createMultiView(FieldPlan.MIME_TYPE, FieldPlanNode.this);
            tc.open();
            tc.requestActive();
        }
    }
}
