/*
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 * 
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *          http://www.eclipse.org/legal/epl-v10.html
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.farm.fieldplan;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import javax.swing.Action;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JToolBar;
import nl.cloudfarming.client.farm.model.FieldPlan;
import nl.cloudfarming.client.util.swing.ASBodyPanel;
import org.netbeans.core.spi.multiview.CloseOperationState;
import org.netbeans.core.spi.multiview.MultiViewElement;
import org.netbeans.core.spi.multiview.MultiViewElementCallback;
import org.openide.awt.UndoRedo;
import org.openide.util.ImageUtilities;
import org.openide.util.Lookup;
import org.openide.util.NbBundle.Messages;
import org.openide.windows.TopComponent;

/**
 * element which will be registered as element for the multi view element
 * for the fieldplan mime type.
 *
 * @author Timon Veenstra <monezz@gmail.com>
 */
@MultiViewElement.Registration(displayName = "#Label_FieldPlan_overview_panel",
iconBase = "nl/cloudfarming/client/farm/fieldplan/plant-icon.png",
mimeType = FieldPlan.MIME_TYPE,
persistenceType = TopComponent.PERSISTENCE_NEVER,
preferredID = "FieldPLanOVerview",
position = 100)
@Messages("Label_FieldPlan_overview_panel=Overview")
public class FieldPlanOverviewElement implements MultiViewElement, PropertyChangeListener {

    private static final long serialVersionUID = 1L;
    private JToolBar toolbar = new JToolBar();
    private final ASBodyPanel content;
    private final Lookup lookup;
    private MultiViewElementCallback callback;

    public FieldPlanOverviewElement(Lookup lookup) {
        this.lookup = lookup;
        FieldPlan fieldPlan = lookup.lookup(FieldPlan.class);
        assert fieldPlan != null : "Lookup passed to FieldPlanOverviewElement should contain a FieldPlan object";
        content = new ASBodyPanel(new FieldPlanOverviewPanel(fieldPlan));
        content.setIcon(ImageUtilities.loadImageIcon("nl/cloudfarming/client/farm/fieldplan/plant-icon32.png", true));
        content.setTitle(Bundle.Label_FieldPlan_overview_panel());

    }

    @Override
    public JComponent getVisualRepresentation() {
        return content;
    }

    @Override
    public JComponent getToolbarRepresentation() {
        return toolbar;
    }

    @Override
    public Action[] getActions() {
        return new Action[0];
    }

    @Override
    public Lookup getLookup() {
        return lookup;
    }

    @Override
    public void componentOpened() {
    }

    @Override
    public void componentClosed() {
    }

    @Override
    public void componentShowing() {
        if (callback != null) {
            updateName();
        }
    }

    @Override
    public void componentHidden() {
    }

    @Override
    public void componentActivated() {
    }

    @Override
    public void componentDeactivated() {
    }

    @Override
    public UndoRedo getUndoRedo() {
        return UndoRedo.NONE;
    }

    @Override
    public void setMultiViewCallback(MultiViewElementCallback callback) {
        this.callback = callback;
    }

    @Override
    public CloseOperationState canCloseElement() {
        return CloseOperationState.STATE_OK;
    }

    @Override
    public void propertyChange(PropertyChangeEvent evt) {
    }

    public void updateName() {
        if (callback != null) {
            String name = getLookup().lookup(FieldPlan.class).getName();
            TopComponent tc = callback.getTopComponent();
            tc.setHtmlDisplayName(name);
            tc.setDisplayName(name);
            tc.setName(content.getName());
            tc.setToolTipText(content.getToolTipText());
        }
    }
}
