/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.farm.fieldplan;

import nl.cloudfarming.client.farm.model.FieldPlanManager;
import java.awt.Image;
import java.awt.datatransfer.Transferable;
import java.beans.BeanInfo;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import javax.swing.Action;
import nl.cloudfarming.client.farm.model.FieldPlan;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataFolder;
import org.openide.nodes.FilterNode;
import org.openide.nodes.Node;
import org.openide.util.Exceptions;
import org.openide.util.ImageUtilities;
import org.openide.util.datatransfer.PasteType;
import org.openide.util.lookup.Lookups;
import org.openide.util.lookup.ProxyLookup;

/**
 * Container node for fieldplans
 *
 * @author Timon Veenstra <monezz@gmail.com>
 */
public class FieldPlansNode extends FilterNode {

    public FieldPlansNode(DataFolder fieldPlansFolder) throws IOException {
        super(fieldPlansFolder.getNodeDelegate(),
                new FieldPlanChildren(fieldPlansFolder),
                new ProxyLookup(fieldPlansFolder.getLookup(), Lookups.fixed(new FieldPlanManagerImpl(fieldPlansFolder))));
    }

    @Override
    public Action[] getActions(boolean context) {
        return new Action[]{new NewFieldPlanAction(getLookup().lookup(FieldPlanManager.class))};
    }

    @Override
    public boolean canCopy() {
        return false;
    }

    @Override
    public boolean canCut() {
        return false;
    }

    @Override
    public PasteType getDropType(Transferable t, int action, int index) {
        return null;
    }

    @Override
    public Image getIcon(int type) {
        switch (type) {
            case BeanInfo.ICON_COLOR_32x32:
            case BeanInfo.ICON_MONO_32x32:
                return ImageUtilities.loadImage("nl/cloudfarming/client/farm/fieldplan/folder-plant-icon32.png");
            default:
                return ImageUtilities.loadImage("nl/cloudfarming/client/farm/fieldplan/folder-plant-icon.png");
        }
    }

    @Override
    public Image getOpenedIcon(int type) {
        return getIcon(type);
    }

    public static class FieldPlanChildren extends FilterNode.Children {

        private final DataFolder folder;

        public FieldPlanChildren(DataFolder folder) {
            super(folder.getNodeDelegate());
            this.folder = folder;
        }

        @Override
        protected Node[] createNodes(Node object) {
            List<Node> result = new ArrayList<>();
            for (Node node : super.createNodes(object)) {
                FileObject fpFolderObject = node.getLookup().lookup(FileObject.class);
                //
                // check if:
                // - child file object is not null
                // - child is a folder
                // - child contains a fieldplan data object
                if (fpFolderObject != null && fpFolderObject.isFolder() && fpFolderObject.getFileObject(FieldPlanManagerImpl.FIELDPLAN_OBJECT_NAME) != null) {
                    FieldPlanManager fieldPlanManager;
                    try {
                        fieldPlanManager = new FieldPlanManagerImpl(folder);
                        FieldPlan fieldPlan = fieldPlanManager.loadFieldPlan(fpFolderObject);
                        if (fieldPlan != null) {
                            result.add(new FieldPlanNode(fieldPlan, node));
                        }
                    } catch (IOException ex) {
                        Exceptions.printStackTrace(ex);
                    }
                }
            }
            return result.toArray(new Node[0]);
        }
    }
}
