/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.farm.fieldplan;

import nl.cloudfarming.client.farm.model.FieldPlanManager;
import java.io.IOException;
import java.util.HashSet;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;
import nl.cloudfarming.client.farm.model.FarmManager;
import org.netbeans.api.project.Project;
import org.netbeans.spi.project.ui.support.NodeFactory;
import org.netbeans.spi.project.ui.support.NodeFactorySupport;
import org.netbeans.spi.project.ui.support.NodeList;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataFolder;
import org.openide.nodes.Node;
import org.openide.util.Exceptions;
import org.openide.util.Lookup;

/**
 *
 * @author Timon Veenstra <monezz@gmail.com>
 */
@NodeFactory.Registration(projectType = "nl-cloudfarming-client-farm")
public class FieldPlansNodeFactory implements NodeFactory {

    static String REGISTERED_NODE_LOCATION = "Projects/nl-cloudfarming-client-farm/Nodes";
    private static final Logger LOG = Logger.getLogger("nl.cloudfarming.client.farm");

    @Override
    public NodeList createNodes(Project project) {
        Set<Node> nodes = new HashSet<>();
        // Add fieldplan nodes
        try {
            FileObject fieldPlansFolder = project.getProjectDirectory().getFileObject(FieldPlanManagerImpl.FARM_FIELDPLANS_DIR);
            FieldPlanManager fieldPlanManager = new FieldPlanManagerImpl(DataFolder.findFolder(fieldPlansFolder));
            FileObject fieldPlanDir = fieldPlanManager.getFieldPlansFolder();
            DataFolder fieldPlanDataFolder = DataFolder.findFolder(fieldPlanDir);
            nodes.add(new FieldPlansNode(fieldPlanDataFolder));
        } catch (IOException ex) {
            LOG.log(Level.SEVERE, "failed to create fieldplans node for farm project {0}", ex.getMessage());
            Exceptions.printStackTrace(ex);
        }


        return NodeFactorySupport.fixedNodeList(nodes.toArray(new Node[nodes.size()]));
    }
}
